//+------------------------------------------------------------------+
//|                                         PolyFitDerivative_v2.mq4 |
//|                             Copyright  2007-09, TrendLaboratory |
//|            http://finance.groups.yahoo.com/group/TrendLaboratory |
//|                                   E-mail: igorad2003@yahoo.co.uk |
//+------------------------------------------------------------------+
#property copyright "Copyright  2007-09, TrendLaboratory"
#property link      "http://finance.groups.yahoo.com/group/TrendLaboratory"

#property indicator_separate_window
#property indicator_buffers 2
#property indicator_color1  SpringGreen
#property indicator_width1  2
#property indicator_color2  OrangeRed
#property indicator_width2  1
#property indicator_style2  2
#property indicator_level1  0
//---- inputs
extern int Order           =   1;   //Order of derivative
extern int Price           =   0;   //Apply to Price(0-Close;1-Open;2-High;3-Low;4-Median price;5-Typical price;6-Weighted Close)
extern int Length          =  14;   //Number of bars for an evaluation
extern int Degree          =   2;   //Degree of a Polynomial(no more 12)
extern int PreSmooth       =   1;   //Period of Preliminary Smoothing
extern int PreSmoothMode   =   0;   //Mode of MA:0-SMA,1-EMA,2-Wilders(SMMA),3-LWMA
extern int Signal          =   5;   //Period of Signal MA
extern int SignalMode      =   3;   //Mode of Signal MA:0-SMA,1-EMA,2-Wilders(SMMA),3-LWMA,4-SineWMA,5-TriMA,6-LSMA,7-SMMA,8-MMA,9-HMA,10-ZeroLagMA
extern int FitMode         =   1;   //Fitting Mode: 0-Fitting,1-Moving
extern int CountBars       =1000;   //Maximum Number of Bars(0-all Bars)

extern bool   alertsOn        = true;
extern bool   alertsOnCurrent = true;
extern bool   alertsMessage   = true;
extern bool   alertsSound     = true;
extern bool   alertsNotify    = false;
extern bool   alertsEmail     = false;
extern string soundfile       = "alert2.wav";

extern bool   ShowArrows      = true;
extern string arrowsIdentifier= "PFDerv Arrows1";
extern color  arrowsUpColor   = LimeGreen;
extern color  arrowsDnColor   = Red;

//---- indicator buffers
double Poly[];
double Sig[];
double trend[];

//---- global variables
double PriceArray[];
int cBars;
//+---------------------------------------------------------------------------+
//| Custom indicator initialization function                                  |
//+---------------------------------------------------------------------------+
int init()
{
   IndicatorDigits(MarketInfo(Symbol(), MODE_DIGITS));
   IndicatorBuffers(3);
   SetIndexStyle(0, DRAW_LINE);
   SetIndexStyle(1, DRAW_LINE);
   SetIndexBuffer(0, Poly);
   SetIndexBuffer(1, Sig);
   SetIndexBuffer(2, trend);
   
   if(CountBars == 0) cBars = Bars - (Length+PreSmooth+Signal-2);
   else cBars = CountBars+1;
   
   SetIndexDrawBegin( 0,Bars-cBars-2);
   SetIndexDrawBegin( 1,Bars-cBars-2);
   
   string short_name="PolyFitDerivative("+Order+","+Price+","+Length+","+Degree+")";
   IndicatorShortName(short_name);
   SetIndexLabel(0,"PolyFitDerivative");
   SetIndexLabel(1,"Signal");
   
   ArrayResize(PriceArray, Length);
   
   return(0);
}

int deinit() 
{ 

   if (ShowArrows) deleteArrows();

return(0); 
}


double PolyFit(double price[],int deg,int len,int bar)
{
double   result=0,Sum=0;
double   AX[13,13],BX[13],CX[13],ZX[13],Pow[26];
int      Row[13];  

   if (len<=1)	Sum=0;
   else
   {	
   
   for (int j=1;j<=deg+1;j++) BX[j]=0;
		
		for (int k=1;k<=len;k++) 
  	   {
  	   double YK=price[len-k];
  	   double XK=k;
  	   double Prod=1;
  	      for (j=1;j<=deg+1;j++)
    	   {
    	   BX[j]+= YK*Prod;
    	   Prod*=XK;
  		   }
      }
      
   for (j=0;j<=2*deg;j++) Pow[j]=0;
	
	Pow[0]=len;
	
      for (k=1;k<=len;k++)
	   {
  	   XK=k;
  	   Prod=k;
  	      for (j=1;j<=2*deg;j++)
  	      {
    	   Pow[j]+= Prod;    	   
    	   Prod*= XK;
  		   }
      }	
	
	   for (j=1;j<=deg+1;j++)
	   {	
         for (int l=1;l<=deg+1;l++) 
  	      AX[j,l]=Pow[j+l-2];
      }	

	for (j=1;j<=deg+1;j++) Row[j]=j;
  	
      for (int i=1;i<=deg;i++)
  	   {
  	      for (k=i+1;k<=deg+1;k++)
         {
            if(MathAbs(AX[Row[k],i]) > MathAbs(AX[Row[i],i]))
      	   {	
      	   int Temp=Row[i];
      	   Row[i]=Row[k];
      	   Row[k]=Temp;
            }
         }

         for (k=i+1;k<=deg+1;k++)
    	   {
    	   if(AX[Row[i],i]!=0) AX[Row[k],i]=AX[Row[k],i]/AX[Row[i],i];
    		   for (l=i+1;l<=deg+1;l++)
      	   AX[Row[k],l]=AX[Row[k],l]-AX[Row[k],i]*AX[Row[i],l];
    		}
		}

   ZX[1]=BX[Row[1]];
      
      for (k=2;k<=deg+1;k++)
  	   {
  	   Sum=0;
      for (l=1;l<=k-1;l++) Sum+=AX[Row[k],l]*ZX[l];
  	   
  	   ZX[k]=BX[Row[k]]-Sum;
	   }
     
	if(AX[Row[deg+1],deg+1] != 0) CX[deg+1]=ZX[deg+1]/AX[Row[deg+1],deg+1];
      	
      for (k=deg;k>=1;k--)
  	   {
  	   Sum=0;
  	   for (l=k+1;l<=deg+1;l++) Sum += AX[Row[k],l]*CX[l]; 
  	   CX[k]=(ZX[k]-Sum)/AX[Row[k],k];
	   }
    
   int ord = Order;
   int mult = 1;
   
   if (ord >= 1)
      for (k = 1; k<=ord; k++) mult *= (deg-k+1); 

	Sum= mult*CX[deg+1];
	
	if(deg > ord) 
	   for (k=deg;k>=ord+1;k--) 
      {
	   int mult1 = 1;
	     if ( ord > 0 )
	     for (i = 1; i<=ord; i++) mult1*=(k-i);
	  
	   Sum = mult1*CX[k]+Sum*(len+bar);
      }         
   }
   return (Sum);
}	
//+------------------------------------------------------------------+
//| PolyFitDerivative_v2                                             |
//+------------------------------------------------------------------+
int start()
{
   int    i, j, k, m, shift, counted_bars=IndicatorCounted(), limit;
        
   
   if ( counted_bars < 0 )  return(0);
   if ( counted_bars ==0 )  limit=Bars-1; 
         
   if (FitMode==1 && counted_bars > 0) {limit = Bars - counted_bars; int len = Length+PreSmooth;}
   else
   if (FitMode==0) {limit = Length; len = cBars - Length+1;} 
   
   if ( counted_bars < 1 || FitMode==0)
   { 
      for(i=1;i<len;i++)
      { 
      Poly[cBars-i]= EMPTY_VALUE;    
      Sig[cBars-i] = EMPTY_VALUE;
      }
   }   

   for(shift = 0;shift<limit;shift++)
   {
      if(shift < cBars)
      {     
         for(j = 0; j < Length; j++) 
         PriceArray[j] = iMA(NULL,0,PreSmooth,0,PreSmoothMode,Price,shift*FitMode + j);
      
      Poly[shift] = PolyFit(PriceArray,Degree,Length,shift*(FitMode-1));
      }
   }
   
   if ( FitMode == 1)   
      for(shift=limit; shift>=0; shift--)
      {
         if(shift == cBars - Signal)
         {
            switch(SignalMode)
            {
            case 1 : Sig[shift] = SMA(Poly,Signal,shift); break;
            case 2 : Sig[shift] = SMA(Poly,Signal,shift); break;
            case 7 : Sig[shift] = SMA(Poly,Signal,shift); break;
            case 10: Sig[shift] = SMA(Poly,1,shift); break;
            }
         }
         else
         if(shift < cBars - Signal)
         {
            switch(SignalMode)
            {
            case 1 : Sig[shift] = EMA(Poly,Sig,Signal,shift); break;
            case 2 : Sig[shift] = Wilder(Poly,Sig,Signal,shift); break;  
            case 3 : Sig[shift] = LWMA(Poly,Signal,shift); break;
            case 4 : Sig[shift] = SineWMA(Poly,Signal,shift); break;
            case 5 : Sig[shift] = TriMA(Poly,Signal,shift); break;
            case 6 : Sig[shift] = LSMA(Poly,Signal,shift); break;
            case 7 : Sig[shift] = SMMA(Poly,Sig,Signal,shift); break;
            case 8 : Sig[shift] = MMA(Poly,Signal,shift); break;
            case 9 : Sig[shift] = HMA(Poly,Signal,shift); break;
            case 10: Sig[shift] = ZeroLagEMA(Poly,Sig,Signal,shift); break;
            default: Sig[shift] = SMA(Poly,Signal,shift); break;
            }
         }
         trend[shift] = trend[shift+1];
         if (Poly[shift]>Sig[shift]) trend[shift] = 1;
         if (Poly[shift]<Sig[shift]) trend[shift] =-1;
         manageArrow(shift);  
      }     
      
      //
      //
      //
      //
      //
        
      if (alertsOn)
      {
        if (alertsOnCurrent)
             int whichBar = 0;
        else     whichBar = 1;
         
        if (trend[whichBar] != trend[whichBar+1])
        if (trend[whichBar] == 1)
              doAlert("Buy");
        else  doAlert("Sell");       
   }

return(0);
}
//+---------------------------------------------------------------------------+

double SMA(double array[],int per,int bar)
{
   double Sum = 0;
   for(int i = 0;i < per;i++) Sum += array[i+bar];
  
   return(Sum/per);
}                

double EMA(double array1[],double array2[],int per,int bar)
{
   return(array2[bar+1] + 2.0/(1+per)*(array1[bar] - array2[bar+1]));
}

double Wilder(double array1[],double array2[],int per,int bar)
{
   return(array2[bar+1] + 1.0/per*(array1[bar] - array2[bar+1]));
}

double LWMA(double array[],int per,int bar)
{
   double Sum = 0;
   double Weight = 0;
   
      for(int i = 0;i < per;i++)
      { 
      Weight+= (per - i);
      Sum += array[bar+i]*(per - i);
      }
   if(Weight>0) double lwma = Sum/Weight;
   else lwma = 0; 
   return(lwma);
} 

double SineWMA(double array[],int per,int bar)
{
   double pi = 3.1415926535;
   double Sum = 0;
   double Weight = 0;
   double del = 0.5*pi/per;
     
      for(int i = 0;i < per;i++)
      { 
      Weight+= MathSin(0.5*pi - del*i);
      Sum += array[bar+i]*MathSin(0.5*pi - del*i);
      }
   if(Weight>0) double swma = Sum/Weight;
   else swma = 0; 
   return(swma);
}

double TriMA(double array[],int per,int bar)
{
   double sma[];
   int len = MathCeil((per+1)*0.5);
   ArrayResize(sma,len);
   
   for(int i = 0;i < len;i++) sma[i] = SMA(array,len,bar+i);
   double trima = SMA(sma,len,0);
   
   return(trima);
}
 
double LSMA(double array[],int per,int bar)
{   
   double Sum=0;
   for(int i=per; i>=1; i--) Sum += (i-(per+1)/3.0)*array[per-i+bar];
   double lsma = Sum*6/(per*(per+1));
   return(lsma);
}

double SMMA(double array1[],double array2[],int per,int bar)
{
   double Sum = 0;
   for(int i = 0;i < per;i++) Sum += array1[i+bar+1];
         
   double smma = (Sum - array2[bar+1] + array1[bar])/per;
   return(smma);
}                

double MMA(double array[],int per,int bar)
{
   double Slope = 0;

   for (int i = 1;i <= per;i++)
   {
   double Factor = 1 + (2 * (i - 1));
   Slope += (array[bar + i - 1] * ((per - Factor)/2));
   }
   double sma = SMA(array,per,bar);
   double mma = sma + (6 * Slope) / ((per + 1) * per);
   return(mma);
}

double HMA(double array[],int per,int bar)
{
   double tmp[];
   int len =  MathSqrt(per);
   ArrayResize(tmp,len);
   
   for(int i = 0; i < len;i++) tmp[i] = 2*LWMA(array,per/2,bar+i) - LWMA(array,per,bar+i);  
   double hma = LWMA(tmp,len,0); 
     
   return(hma);
}

double ZeroLagEMA(double array1[],double array2[],int per,int bar)
{
   double alfa = 2.0/(1+per); 
   int lag = 0.5*(per - 1); 
   return(alfa*(2*array1[bar] - array1[bar+lag]) + (1-alfa)*array2[bar+1]);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

void doAlert(string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[0]) {
          previousAlert  = doWhat;
          previousTime   = Time[0];

          //
          //
          //
          //
          //

          message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," Polyfit Deravitive ",doWhat);
             if (alertsMessage) Alert(message);
             if (alertsNotify)  SendNotification(message);
             if (alertsEmail)   SendMail(StringConcatenate(Symbol()," Polyfit Derivative "),message);
             if (alertsSound)   PlaySound(soundfile);
      }
}
 
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void manageArrow(int i)
{
   if (ShowArrows)
   {
      deleteArrow(Time[i]);
      if (trend[i]!=trend[i+1])
      {
         if (trend[i] == 1) drawArrow(i,arrowsUpColor,225,false);
         if (trend[i] ==-1) drawArrow(i,arrowsDnColor,226,true);
      }
   }
}               

//
//
//
//
//

void drawArrow(int i,color theColor,int theCode,bool up)
{
   string name = arrowsIdentifier+":"+Time[i];
   double gap  = 3.0*iATR(NULL,0,20,i)/4.0;   
   
      //
      //
      //
      //
      //
      
      ObjectCreate(name,OBJ_ARROW,0,Time[i],0);
         ObjectSet(name,OBJPROP_ARROWCODE,theCode);
         ObjectSet(name,OBJPROP_COLOR,theColor);
         if (up)
               ObjectSet(name,OBJPROP_PRICE1,High[i]+ gap);
         else  ObjectSet(name,OBJPROP_PRICE1,Low[i] - gap);
}

//
//
//
//
//

void deleteArrows()
{
   string lookFor       = arrowsIdentifier+":";
   int    lookForLength = StringLen(lookFor);
   for (int i=ObjectsTotal()-1; i>=0; i--)
   {
      string objectName = ObjectName(i);
         if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(objectName);
   }
}

//
//
//
//
//

void deleteArrow(datetime time)
{
   string lookFor = arrowsIdentifier+":"+time; ObjectDelete(lookFor);
}



     